package com.privatenumber;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

// 外部类库，用于加密
import org.apache.commons.codec.digest.DigestUtils;
import org.json.JSONObject;

/**
 * 隐私号码保护类
 */
public class PrivateNumber {
	// 网站的appkey
	private String appkey = "";
	// appkey 对应的秘钥
	private String secret = "";
	public String apiurl = "http://api.mixcom.cn";

	/**
	 * 设置appkey
	 * @param appkey
	 */
	public void setAppkey(String appkey) {
		this.appkey = appkey;
	}

	/**
	 * 设置秘钥
	 * @param secret
	 */
	public void setSecret(String secret) {
		this.secret = secret;
	}

	/**
	 * 获取服务器的时间
	 * @return
	 */
	public JSONObject getServerTime()
	{
		String url = this.apiurl + "/virtual/getServerTime";
		String json = this.get(url);
		// http://mvnrepository.com/artifact/org.json/json
	    JSONObject obj =  new JSONObject(json);
	    return obj;
	}

	/**
	 * 获取可用的axb号码
	 * @param code code 区号，如深圳区号为 0755， 通过这个参数查询指定地区的axb号码， 可选
	 * @return
	 */
	public JSONObject getAxbNumber(String code)
	{
		String url = this.apiurl + "/virtual/getAxbNumber";
		Map<String, String> map = new HashMap<String, String>();
		map.put("region_code", code);
		return this.sendpost(url, map);
	}

	/**
	 * axb号码绑定
	 * @return
	 */
	public JSONObject axbBind(Map<String, String> map)
	{
		String url = this.apiurl + "/virtual/axbBind";
		return this.sendpost(url, map);
	}

	/**
	 * axb号码解绑
	 * @param number 虚拟小号，需要加86，必选
	 * @param aparty 呼叫方的号码，必选
	 * @param bparty 被叫方的号码，必选
	 * @return
	 */
	public JSONObject unAxbBind(String number, String aparty, String bparty)
	{
		String url = this.apiurl + "/virtual/unAxbBind";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("aparty", aparty);
		map.put("bparty", bparty);
		return this.sendpost(url, map);
	}

	/**
     * axb号码自动绑定
     * @param aparty 呼叫方的号码，必选
     * @param bparty 被叫方的号码，必选
     * @param region_code 需要的虚拟小号的区号, 可选
     * @param record 是否开启录音，1开启， 0 不开启。使用前请先确定使用了通话录音功能
     * @param enddate 解绑时间，毫秒级的时间戳，可选，0代表不设置解绑时间
     * @param maxduration 最长通话分钟数，可选
     * @return
     */
	public JSONObject axbAutoBind(String aparty, String bparty, String region_code, int record, long enddate, int maxduration)
	{
		String url = this.apiurl + "/virtual/axbAutoBind";
		Map<String, String> map = new HashMap<String, String>();
		map.put("aparty", aparty);
		map.put("bparty", bparty);
		map.put("region_code", region_code);
		map.put("recording",  Integer.toString(record));
		map.put("maxduration", Integer.toString(maxduration));
		if(enddate != 0) {
			map.put("enddate", Long.toString(enddate / 1000));
		}
		return this.sendpost(url, map);
	}

	/**
	 * 获取可用的ax号码
	 * @param code 区号，如深圳区号为 0755， 通过这个参数查询指定地区的ax号码， 可选
	 * @return
	 */
	public JSONObject getAxNumber(String code)
	{
		String url = this.apiurl + "/virtual/getAxNumber";
		Map<String, String> map = new HashMap<String, String>();
		map.put("region_code", code);
		return this.sendpost(url, map);
	}

	/**
	 * ax号码绑定
	 * @param number 虚拟小号，需要加86，必选
	 * @param aparty 呼叫方的号码，必选
	 * @param record  是否开启录音，1开启， 0 不开启。使用前请先确定使用了通话录音功能
	 * @param displaycall 是否开启号码透传。默认为0，不开启。1为开启，开启后可能会导致部分通话无法接通
	 * @param enddate 解绑时间，毫秒级的时间戳，可选，0代表不设置解绑时间
	 * @return
	 */
	public JSONObject axBind(String number, String aparty, int record, int displaycall, long enddate)
	{
		String url = this.apiurl + "/virtual/axBind";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("aparty", aparty);
		map.put("recording", Integer.toString(record));
		map.put("display_call", Integer.toString(displaycall));
		if(enddate != 0) {
			map.put("enddate", Long.toString(enddate / 1000));
		}
		return this.sendpost(url, map);
	}

	/**
	 * ax号码解绑
	 * @param number 虚拟小号，需要加86，必选
	 * @param aparty A方的号码，必选
	 * @return
	 */
	public JSONObject unAxBind(String number, String aparty, String bparty)
	{
		String url = this.apiurl + "/virtual/unAxBind";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("aparty", aparty);
		return this.sendpost(url, map);
	}

	/**
	 * AX绑定呼出
	 * @param number 虚拟小号，需要加86，必选
	 * @param aparty 呼叫方的号码，必选
	 * @param bparty 被叫方的号码，必选
	 * @return
	 */
	public JSONObject axDialNumber(String number, String aparty, String bparty)
	{
		String url = this.apiurl + "/virtual/axDialNumber";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("aparty", aparty);
		map.put("bparty", bparty);
		return this.sendpost(url, map);
	}

	/**
     * ax号码自动绑定
     * @param aparty 呼叫方的号码，必选
     * @param region_code 需要的虚拟小号的区号, 可选
     * @param record  是否开启录音，1开启， 0 不开启。使用前请先确定使用了通话录音功能
     * @param displaycall 是否开启号码透传。默认为0，不开启。1为开启，开启后可能会导致部分通话无法接通
     * @param enddate 解绑时间，毫秒级的时间戳，可选，0代表不设置解绑时间
     * @return
     */
	public JSONObject axAutoBind(String aparty, String region_code, int record, int displaycall, long enddate)
	{
		String url = this.apiurl + "/virtual/axAutoBind";
		Map<String, String> map = new HashMap<String, String>();
		map.put("aparty", aparty);
		map.put("region_code", region_code);
		map.put("recording", Integer.toString(record));
		map.put("display_call", Integer.toString(displaycall));
		if(enddate != 0) {
			map.put("enddate", Long.toString(enddate / 1000));
		}
		return this.sendpost(url, map);
	}

	/**
	 * 挂断指定通话
	 * @param subscriptionid 该通话的绑定关系id
	 * @return
	 */
	public JSONObject callStop(String subscriptionid)
	{
		String url = this.apiurl + "/virtual/callStop";
		Map<String, String> map = new HashMap<String, String>();
		map.put("subscriptionid", subscriptionid);
		return this.sendpost(url, map);
	}

	/**
	 * 获取小号的个数
	 * @return
	 */
	public JSONObject countNumber()
	{
		String url = this.apiurl + "/virtual/countNumber";
		Map<String, String> map = new HashMap<String, String>();
		return this.sendpost(url, map);
	}

	/**
	 * 查询所有的号码
	 * @param page 需要查询的页数，每页100条数据
	 * @return
	 */
	public JSONObject queryAllNumber(int page) {
		page = page == 0 ? 1 : page;
		String url = this.apiurl + "/virtual/queryAllNumber";
		Map<String, String> map = new HashMap<String, String>();
		map.put("page", Integer.toString(page));
		return this.sendpost(url, map);
	}

	/**
	 * 查询号码的绑定关系
	 * @param number 虚拟小号，需要加86，可选
	 * @param aparty 呼叫方的号码，可选
	 * @param bparty 被叫方的号码，可选
	 * @return
	 */
	public JSONObject queryBindRelation(String number, String aparty, String bparty)
	{
		String url = this.apiurl + "/virtual/querybindrelation";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("aparty", aparty);
		map.put("bparty", bparty);
		return this.sendpost(url, map);
	}

	/**
     * 查询号码的绑定关系日志
     * @param number 虚拟小号，需要加86，可选
     * @param aparty 呼叫方的号码，可选
     * @param bparty 被叫方的号码，可选
     * @param timestart 开始时间，可选
     * @param timeend 结束时间，可选
     * @param page 页数，可选
     * @return
     */
	public JSONObject queryRelationLog(String number, String aparty, String bparty, long timestart, long timeend, int page)
	{
		String url = this.apiurl + "/virtual/queryrelationlog";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("aparty", aparty);
		map.put("bparty", bparty);
		map.put("timestart", Long.toString(timestart/ 1000) );
		map.put("timeend", Long.toString(timeend/ 1000) );
		map.put("page", Integer.toString(page));
		return this.sendpost(url, map);
	}

	/**
     * 查询通话记录
     * @param number 虚拟小号，需要加86，可选
     * @param aparty 呼叫方的号码，可选
     * @param bparty 被叫方的号码，可选
     * @param timestart 开始时间，可选
     * @param timeend 结束时间，可选
     * @param page 页数，可选
     * @return
     */
	public JSONObject queryCallRecord(String number, String aparty, String bparty, long timestart, long timeend, int page)
	{
		String url = this.apiurl + "/virtual/querycallrecord";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("aparty", aparty);
		map.put("bparty", bparty);
		map.put("timestart", Long.toString(timestart/ 1000) );
		map.put("timeend", Long.toString(timeend/ 1000) );
		map.put("page", Integer.toString(page));
		return this.sendpost(url, map);
	}

	/**
     * 返回录音的长效链接(在录音未删除之前有效)
     * @param number 虚拟小号
     * @param callIdentifier 通话录音id
     */
	public JSONObject getRecordUrl(String number, String callIdentifier)
	{
		String url = this.apiurl + "/virtual/getrecordurl";
		Map<String, String> map = new HashMap<String, String>();
		map.put("number", number);
		map.put("callIdentifier", callIdentifier);
		return this.sendpost(url, map);
	}

	/**
	 * 加入公用的参数发送请求
	 */
	private JSONObject sendpost(String url, Map<String, String> map)
	{
		map.put("time", Long.toString(System.currentTimeMillis()/ 1000) );
		map.put("appkey", this.appkey);
		String sign = this.getSign(map);
		map.put("sign", sign);
		String json = this.post(url, map);
		JSONObject obj =  new JSONObject(json);
	    return obj;
	}

	/**
	 * 发起get请求
	 */
	private String get(String url)
	{
		String resault = "";
		String line = "";
		try {
			String method = "GET";
			HttpURLConnection  conn = this.getConnection(url, method);
			conn.setReadTimeout(5000);
	        conn.setConnectTimeout(5000);
			conn.connect();
			// 获取数据
			BufferedReader read = new BufferedReader(new InputStreamReader(conn.getInputStream(), "utf-8"));
			while((line = read.readLine()) != null) {
				resault += line.toString();
			}
		} catch(Exception e) {
			e.printStackTrace();
		}
		return resault;
	}

	/**
	 * 发起post请求
	 */
	private String post(String url, Map<String, String> params)
	{
		String resault = "";
		String line = "";
		String poststr = "";
		DataOutputStream out = null;
		try {
			String method = "POST";
			HttpURLConnection  conn = this.getConnection(url, method);
			conn.setReadTimeout(5000);
	        conn.setConnectTimeout(5000);
			conn.connect();
			//处理map数据
			for(Map.Entry<String, String> entry : params.entrySet()) {
				poststr += entry.getKey() + "=" + URLEncoder.encode(entry.getValue(), "utf-8") + "&";
			}
			poststr = poststr.substring(0, poststr.length() -1);
			// post请求
			out = new DataOutputStream(conn.getOutputStream());
			out.writeBytes(poststr);
			out.flush();
			out.close();
			// 获取数据
			BufferedReader read = new BufferedReader(new InputStreamReader(conn.getInputStream(), "utf-8"));
			while((line = read.readLine()) != null) {
				line = new String(line.getBytes(), "utf-8");
				resault += line.toString();
			}
			read.close();
			conn.disconnect();
		} catch(Exception e) {
			e.printStackTrace();
		}
		return resault;
	}

	/**
	 * 获取连接
	 */
	private HttpURLConnection getConnection(String url, String method)
	{
		HttpURLConnection  connection = null;
		try {
			URL httpurl = new URL(url);
			connection = (HttpURLConnection)httpurl.openConnection();
			if(method == "GET") {
				connection.setDoOutput(false);
			} else {
				connection.setDoOutput(true);
			}
			connection.setDoInput(true);
			connection.setUseCaches(false);
			connection.setRequestMethod(method);
			connection.setInstanceFollowRedirects(true);
			connection.setRequestProperty("content-type", "application/x-www-form-urlencoded");
			connection.setRequestProperty("Charset", "utf-8");
			connection.setRequestProperty("Accept-Charset", "utf-8");
		} catch(Exception e) {
			e.printStackTrace();
		}
		return connection;
	}

	/**
	 * 生成加密检验码
	 * @param params
	 * @return
	 */
	private String getSign(Map<String, String> params) {
		Iterator<Map.Entry<String, String>> it = params.entrySet().iterator();
		while(it.hasNext()) {
			Map.Entry<String, String> entry = it.next();
			if(entry.getValue() == "") {
				it.remove();
			}
			if(entry.getKey() == "sign") {
				it.remove();
			}
			if(entry.getKey() == "a") {
				// 加密的时候a会排除，请勿使用
				it.remove();
			}
			if(entry.getKey() == "c") {
				// 加密的时候c会排除，请勿使用
				it.remove();
			}
		}

		// map排序
		List<Map.Entry<String, String>> list = new ArrayList<Map.Entry<String, String>>(params.entrySet());
		Collections.sort(list, new Comparator<Map.Entry<String, String>>(){
			public int compare(Map.Entry<String, String> s1, Map.Entry<String, String> s2) {
				return s1.getKey().compareTo(s2.getKey());
			}
		});
		// 生成sign
		String str = new String();
		for(Map.Entry<String,String> mapping:list){
			str += mapping.getValue();
        }
		str += this.secret;
		// 需要导入外部的类库
		// http://commons.apache.org/proper/commons-codec/
		// http://search.maven.org/remotecontent?filepath=commons-codec/commons-codec/1.11/commons-codec-1.11.jar
		// 如果sha1Hex无法使用，请尝试使用shaHex. shaHex为已过期的方法，推荐使用sha1Hex
		String sign = DigestUtils.sha1Hex(str + DigestUtils.sha1Hex(this.secret)).toLowerCase();
		return sign;
	}
}
